/*
 * Threads.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>		// strlen()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
//#include <time.h>
#include <sys/time.h>
#include <pthread.h>
#include <semaphore.h>
#include <sys/poll.h>
#include "gefcmn_gpio.h"
#include "seedcore_api.h"

#define BUFF_SIZE 2000
#define TX_SIZE0 352
#define TX_SIZE1 202
#define TX_SIZE2 120
#define TX_SIZE3 256
#define LOG_TX_SIZE0 (352 * 1024)
#define LOG_TX_SIZE1 (202 * 1024)
#define LOG_TX_SIZE2 (120 * 1024)
#define LOG_TX_SIZE3 (256 * 1024)
//#define BUFF_SIZE (1048*80)
#define WRITE_COUNT 1
#define LOG_RX_SIZE0 (352 * 1024)
#define LOG_RX_SIZE1 (202 * 1024)
#define LOG_RX_SIZE2 (120 * 1024)
#define LOG_RX_SIZE3 (256 * 1024)
#define NUM_OF_CH 4

extern int fd[4];
extern GEF_GPIO_HDL gpioHdl;

int num_txcall[4];
sem_t g_sendSem[NUM_OF_CH];

static unsigned int frame_size[4] = {0,0,0,0};
static unsigned long int g_frame_size[4] = {0,0,0,0};


/* get the current time with millisecond since the Epoch(1970-1-1). */
unsigned int GetMSec()
{
	unsigned int uiCurTime;
	struct timeval tv;

	gettimeofday(&tv, NULL);

	uiCurTime = tv.tv_sec * 1000 + tv.tv_usec/1000;

	return uiCurTime;
}


int iwannaquit= 0;

void *timer_thread(void *arg)
{
	unsigned int uiStartTime = 0;
	unsigned int uiCurrTime = 0;
	int iTickInterVal = 1000;
	int uiTickCount = 0;
	int ch = 0;

	uiStartTime = GetMSec();

	while( iwannaquit==0 ) {
		int rc = getchar();
		for(ch=0; ch <4; ch++) {
			printf("CH[%d] NUM send call %d \n", ch, num_txcall[ch]);
//			sem_post(&g_sendSem[ch]);
		}
		printf("getchar value(rc): %d \n",rc);

		if( rc!=10 ) { 
			iwannaquit = 1;
			printf("iwannaquit is set to 1. \n");
		}

		printf("\n");
//		sleep(1);
	}
	printf("bye \n");

#if 0
	while(1)
	{
		uiCurrTime = GetMSec();
		if( (uiCurrTime - uiStartTime) >= iTickInterVal )
		{
			uiTickCount++;
//			printf("[%d] Rx[%d] %5.3f Gbps (%ld)\n", uiTickCount, ch, g_frame_size[ch] * 8   / 1000000000.f, frame_size[ch]);
		//	printf("[%d] Rx[0] %5.3f Gbps (%ld), Rx[1] %5.3f Gbps (%ld), Rx[2] %5.3f Gbps (%ld), Rx[3] %5.3f Gbps (%ld)\n", uiTickCount, g_frame_size[0] * 8   / 1000000000.f, frame_size[0], g_frame_size[1] * 8   / 1000000000.f, frame_size[1], g_frame_size[2] * 8   / 1000000000.f, frame_size[2], g_frame_size[3] * 8   / 1000000000.f, frame_size[3]);
			g_frame_size[0] = 0;
			g_frame_size[1] = 0;
			g_frame_size[2] = 0;
			g_frame_size[3] = 0;
			uiStartTime = uiCurrTime;
		}
		usleep(100000);
	}
	return NULL;
#endif
}


void *gpio_thread( void* arg)
{
	int i = 0;
	int pin = 1;
	GEF_GPIO_PIN_BITMAP pinBitmap = (0x0001 << pin);
	GEF_STATUS gpioStatus;
	struct timeval timeout;

	timeout.tv_sec = 3;
	timeout.tv_usec = 0;

	while(1) {
		/* waiting for GIOP interrupt, after timeout, it's released. */
		gefGpioWaitForInterrupt( gpioHdl, &pinBitmap, &timeout);
//		printf("GPIO(pin:%d) interrupted or timeout.\n", pin );

		for( i=0; i < NUM_OF_CH; i++)	sem_post( &g_sendSem[i] );
		if( iwannaquit==1 ) break;
	}
	return NULL;
}


void *send_thread(void* arg)
{
	int ch = 0;
	unsigned int buff[BUFF_SIZE];
	unsigned int i, j = 0;
	unsigned int pattern = 0;
	int rtn=0, cnt=0;
	int logfp;
	char logFile[80];
	int logstopflag = 0;
	int data_txsize, log_txsize = 0;
	unsigned int ret_size = 0;
	unsigned int ret = 0;
	int count = 0;
#if 0
	int pin = 1;
	GEF_GPIO_PIN_BITMAP pinBitmap = (0x0001 << pin);
	GEF_STATUS gpioStatus;
#endif
	ch =  *(int *) arg;

	printf("Send_thread[%d] starts.\n", ch);
	memset(buff, pattern, BUFF_SIZE);

	for(i=0; i < BUFF_SIZE ; i++) {
		buff[i] = pattern;
		pattern++;
	}

	if (ch == 0) sprintf(logFile, "log-send0");
	else if((ch >= 1) && (ch <= 3))
		sprintf(logFile, "log-send%d", ch);
	else {
		printf("[%s] Invalid ch number %d\n", __FUNCTION__, ch);
		return NULL;
	}

	if( !(logfp = open(logFile, O_WRONLY | O_CREAT, 0644)))
		printf("logFile Open Err\n");

	if(ch == 0) {
		data_txsize = TX_SIZE0;
		log_txsize = LOG_TX_SIZE0;
		buff[0]=0xB1C1;
	}
	else if(ch == 1) {
		data_txsize = TX_SIZE1;
		log_txsize = LOG_TX_SIZE1;
		buff[0]=0xB1C2;
	}
	else if(ch == 2) {
		data_txsize = TX_SIZE2;
		log_txsize = LOG_TX_SIZE2;
		buff[0]=0xB1C3;
	}
	else if(ch == 3) {
		data_txsize = TX_SIZE3;
		log_txsize = LOG_TX_SIZE3;
		buff[0]=0xB1C4;
	}

//	while( iwannaquit == 0 ) {
	for( i=0; i<2; i++ ) {
		rtn = sem_wait( &g_sendSem[ch] );	/* wait for the semaphore */
		if( rtn<0 ) break; 
		if( iwannaquit == 1 ) break;

		for(cnt = 0; cnt < WRITE_COUNT; cnt++) {
			buff[1] = j++;

			rtn = sdlc_send( ch, buff, data_txsize );
			if( rtn < 0 ) {
				printf("[%s] ch(%d) send fail..\n", __FUNCTION__, ch);
				usleep(1000000);
			}
			else printf("Send data(size:%d) through ch:%d.\n", rtn, ch);

			num_txcall[ch]++;
/*
			if( !logstopflag ) write( logfp, buff, log_txsize);
*/
			if( !logstopflag ) {
				write( logfp, buff, rtn);
				ret_size += rtn;
				if( ret_size >= log_txsize ) logstopflag = 1;
			}

			usleep(50);
		}
//		logstopflag = 1;
		gefGpioWrite( gpioHdl, SBC624_GPIO_PIN_7 ); 
		gefGpioWrite( gpioHdl, SBC624_GPIO_RESET ); 
	}
	iwannaquit = 1; 

	printf("[%s] Quit..\n", __FUNCTION__);
	return NULL;
}


void *recv_thread(void* arg)
{
	int ch = 0;
	struct pollfd poll_events;
	int poll_state;
	unsigned int buff[BUFF_SIZE];
	int ret, i, rtn = 0;
	int logfp;
	char logFile[80];
	int logstopflag = 0;
	unsigned int ret_size = 0;
	unsigned int log_rx_size = 0;

	ch = *(int *)arg;

	poll_events.fd        = fd[ch];
	poll_events.events    = POLLIN | POLLERR;	// requested events: 수신 자료 or 에러가 있는지
	poll_events.revents   = 0;			// returned events.

	printf("Recv_thread[%d] \n", ch);

	if (ch == 0)	sprintf(logFile, "log-recv0");
	else if((ch >= 1) && (ch <= 3)) sprintf(logFile, "log-recv%d", ch);
	else {
		printf("[%s] Invalid ch number %d\n", __FUNCTION__, ch);
		return NULL;
	}

	if (!(logfp = open(logFile, O_WRONLY | O_CREAT, 0644)))
		printf("logFile Open Err\n");

	if(ch == 0)	log_rx_size = LOG_RX_SIZE0;
	else if(ch == 1) log_rx_size = LOG_RX_SIZE1;
	else if(ch == 2) log_rx_size = LOG_RX_SIZE2;
	else if(ch == 3) log_rx_size = LOG_RX_SIZE3;

	while(1) {
		poll_state = poll(		// It calls pci_poll(), events 발생 확인
				(struct pollfd*) &poll_events, // possible several fds. 
				1,		// 체크할 pollfd 개수
				5000 );		// time out 시간: millisec.
		printf("[%s] Polling or timeout in ch[%d]. state: %d\n", __FUNCTION__, ch, poll_state );

		if( (0<poll_state) && (poll_events.revents & POLLIN) ) {
			// Event occurs and data exist. 발생한 event가 있음 && event가 자료수신. 
			while( 1 ) {
				ret = sdlc_recv( ch, buff, BUFF_SIZE);
				if( ret>0 ) printf("Receive Data(size:%d) though ch:%d.", ret, ch );
				else	break;	/* there is not the received data */ 

				frame_size[ch] = ret;
				g_frame_size[ch] += frame_size[ch];
#if 0
				for( i=501; i<520; i++ ) {
					printf("CH[%d] len[%d] recv %d[0x%08x]  ", ch, ret, i, buff[i]);
					if( (i%5) == 0 ) printf("\n");
				}
				printf("\n\n");
#endif
			//	printf("ch[%d] len[%d] %d[0x%08x] %d[0x%08x] %d[0x%08x] %d[0x%08x] %d[0x%08x] %d[0x%08x] %d[0x%08x] %d[0x%08x]\n", ch, ret, 23, buff[23], 24, buff[24], 48, buff[48], 49, buff[49], 249, buff[249], 250, buff[250], 499, buff[499], 500, buff[500] );

				if( !logstopflag ) {
					write( logfp, buff, ret);
					ret_size += ret;
					if(ret_size >= log_rx_size) logstopflag = 1;
				}
				memset(buff, 0, BUFF_SIZE*4);
			}
		}

		else if( poll_events.revents & POLLERR ) {	// event 가 에러?
				printf( "[%s] poll_events error..\n", __FUNCTION__ );
				break;
		}
		
	} //while

	return NULL;
}


int main(int argc, char **argv)
{
	int chNum[4] = {0, 1, 2, 3};
	int semIndex = 0;

	pthread_t p_send0;
	pthread_t p_send1;
	pthread_t p_send2;
	pthread_t p_send3;

	pthread_t p_recv0;
	pthread_t p_recv1;
	pthread_t p_recv2;
	pthread_t p_recv3;
	pthread_t p_tick_thread;
	pthread_t p_gpio;

	int iSendThreadID0;
	int iSendThreadID1;
	int iSendThreadID2;
	int iSendThreadID3;

	int iRecvThreadID0;
	int iRecvThreadID1;
	int iRecvThreadID2;
	int iRecvThreadID3;
	int iTickThreadID;
	int iGpioThreadID;

	system("rm -f log*");

	scGpioInit();

	for( semIndex = 0; semIndex < NUM_OF_CH; semIndex++) {
		if( sem_init( &g_sendSem[semIndex], 0, 0) == -1) {
			printf("[%s] Semaphore ERROR..\n", __FUNCTION__);
			return -1;
		}
	}

	sdlc_init();	/* open device files */

	iGpioThreadID = pthread_create( &p_gpio, NULL, gpio_thread, NULL);
	iTickThreadID = pthread_create( &p_tick_thread, NULL, timer_thread, NULL);

	iSendThreadID0 = pthread_create( &p_send0, NULL, send_thread, (void *)&chNum[0]);
/*
	iSendThreadID1 = pthread_create( &p_send1, NULL, send_thread, (void *)&chNum[1]);
	iSendThreadID2 = pthread_create( &p_send2, NULL, send_thread, (void *)&chNum[2]);
	iSendThreadID3 = pthread_create( &p_send3, NULL, send_thread, (void *)&chNum[3]);

	iRecvThreadID0 = pthread_create( &p_recv0, NULL, recv_thread, (void *)&chNum[0]);
*/
	iRecvThreadID1 = pthread_create( &p_recv1, NULL, recv_thread, (void *)&chNum[1]);
/*
	iRecvThreadID2 = pthread_create( &p_recv2, NULL, recv_thread, (void *)&chNum[2]);
	iRecvThreadID3 = pthread_create( &p_recv3, NULL, recv_thread, (void *)&chNum[3]);
*/
	pthread_join(p_send0, NULL);

#if 0
	while( iwannaquit == 0 );

	pthread_join(p_send1, NULL);
	pthread_join(p_send2, NULL);
	pthread_join(p_send3, NULL);
//	pthread_join(p_recv0, NULL);
//	pthread_join(p_recv1, NULL);
//	pthread_join(p_recv2, NULL);
//	pthread_join(p_recv3, NULL);

	pthread_join(p_tick_thread, NULL);
#endif
	scGpioClose();
	sdlc_close();
	return 0;
}


