/*
 * send & recv message through a HDLC card. 
 *
 */
#include <stdio.h>
#include <stdlib.h>		// atoi()
#include <pthread.h>
#include <string.h>		// memcpy()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <time.h>
#include <semaphore.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/poll.h>

#include "gefcmn_gpio.h"
#include "hdlc-api.h"


/*#define TXBUF_SIZE	(16*1024) 	// 16KB 
*/

extern int scGpioInit();
extern int scGpioSet();
extern int scGpioClose();
extern int scGpioSetHigh6(void);
extern int scGpioSetLow6(void);
extern int scGpioSetHigh7(void);
extern int scGpioSetLow7(void);

extern GEF_GPIO_HDL gpioHdl;

static int TXsize=4096;

sem_t sendSem[NUM_OF_DMA];


void *gpio_thread(void* arg)
{
	int i = 0;
	int inpin = 0;		// Intrpt GPIO input pin #0: MCP
	GEF_GPIO_PIN_BITMAP pinBitmap;
	GEF_STATUS gpioStatus;
	struct timeval timeout;

	pinBitmap = (0x0001 << inpin);
	timeout.tv_sec = 2;	// timeout is 3 seconds
	timeout.tv_usec = 0;

	while(1) {
		getchar();
		gefGpioWaitForInterrupt( gpioHdl, &pinBitmap, &timeout );

		printf("GPIO(pin#%d) Interrupted or timeout(%d sec)(%d usec)..\n", inpin, timeout.tv_sec ,timeout.tv_usec);
/*
//		scGpioSetHigh7();
//		scGpioSetLow7();
//		SleepUSec(400*1);
*/
		for( i=0; i < NUM_OF_DMA; i++ )	sem_post( &sendSem[i] );
	}
	return NULL;
}


/* It sends the data with ? size through the AIM ch (="arg"). */
void *send_thread(void* arg)
{
	int ch=0;
	int i; 
	int rtn=0;
	int totsend=0;		// total size of send
	int cnt = 0;
	unsigned int	chksum=0;		// for packet data integrity 
	unsigned char	tbuf[TX_BUFSIZE+64];	// 16 KB
	int count = 0;

	ch = *(int *)arg;
	srand( time(NULL) );

	while(1) {
		sem_wait( &sendSem[ch] );

		chksum = 0;
/*		tbuf[0] = 0xAA;
		tbuf[1] = 0x55;
		tbuf[2] = 0xAA;
		tbuf[3] = 0x55;
*/
		for(i=0; i < (TXsize-sizeof(int)); i++) {
			tbuf[i] = i+1;
//			tbuf[i] = rand() % 10;
			chksum += tbuf[i];
		}

		memcpy( (void*)&tbuf[i], (void*)&chksum, sizeof(int) );

		rtn = hdlc_send( ch, (char *)tbuf, TXsize );
		if( !(0<rtn) ) {
			printf( "%d-th block, there is no data of hdlc_send.\n", cnt );
		}
		printf( "CH[%d] TX size: %d Bytes (Org TXsize:%d)\n", ch, rtn, TXsize );
		totsend += rtn;

		for( i=0; i< TXsize; i++) {
			if( (i%16)== 0 ) printf( "tbuf[%3d] ", i );
			printf( "%02X ", tbuf[i] ); 
			if( (i%16)== 15 ) printf( "\n" );
		}
		printf( "\n" );

		printf("Ch[%d] Total TX Size: %d Bytes\n", ch, totsend );
		printf("Ch[%d] TX Check-sum value: 0x%X\n", ch, chksum );

	} // while

	return NULL;
}



void *recv_thread(void* arg)
{
	int ch=0, i;
	int nIndex;
	int cntblk=0, retlen; 
	int totrecv;
	unsigned char	rbuf[RX_BUFSIZE+16];
	unsigned int	calchksum, rxchksum;

	ch = *(int *)arg;
	totrecv=0;

	while(1) {

		retlen = hdlc_recv( ch, (char *)rbuf, TXsize, 0);
		if( retlen<0 ) {
			printf("\nERR: channel link RX err, retlen: %d\n", retlen );
			return NULL; 
		}

		printf("\nCH[%d] RX size: %d Bytes\n", ch, retlen );
		for( i=0; i< (retlen); i++) {
			if( (i%16)== 0 ) printf( "rbuf[%3d]", i );
			printf( " %02X", rbuf[i] ); 
			if( (i%16)== 15 ) printf( "\n" );
		}
		printf( "\n" );
			
/*		printf("rbuf[0] =0x%02X, rbuf[1] =0x%02X ", rbuf[0], rbuf[1] );
		printf("rbuf[4]cnt= 0x%02X \n", rbuf[4]);
*/

		calchksum = 0;
		for(i=0; i< (retlen-sizeof(int)); i++) {	// to calcurate check-sum of RX data
			calchksum += rbuf[i];
		}

		memcpy( (void*)&rxchksum, (void*)&rbuf[retlen-sizeof(int)], sizeof(int) );
		if( calchksum != rxchksum ) {
			printf(">>>Err: Calculated chksum=0x%X, RX chksum=0x%X\n", calchksum, rxchksum); 
		}

		totrecv += retlen; 
		printf("Ch[%d] RX Total size: %d Bytes\n", ch, totrecv );
		printf("Calculated chksum: 0x%X, RX chksum: 0x%X\n", calchksum, rxchksum ); 

	} /*while*/

	return NULL;
}



int get_main_arg( int argc, char **argv )
{
        int chno;

        if( argc != 3 ) printf( "Usage: %s [ch#] [TXsize]\n\n", argv[0] );
        if( argc > 3 ) return -1;

        while(1) {
                if( argc == 1 ) {
                        printf( "Select the channel# and size to recieve data..\n" );
                        printf( "Range of the Channel number is from 0 to %d: ", NUM_OF_DMA );
                        scanf( "%d", &chno );
                        printf( "Size of Tx data (5 ~ %d Byte): ", TX_BUFSIZE );
                        scanf( "%d", &TXsize );
                }
                else if( argc == 2 ) {
                        chno = atoi( argv[1] );
                        printf( "Size of Tx data (5 ~ %d Byte): ", TX_BUFSIZE );
                        scanf( "%d", &TXsize );
                }
                else if( argc == 3 ) {
                        chno = atoi( argv[1] );
                        TXsize = atoi( argv[2] );
                }

                if( (chno<0) || (chno>NUM_OF_DMA) || (TXsize<0) || (TXsize >TX_BUFSIZE) ) {
                        printf( "Range of Input is over: Ch[%d], size:%d\n", chno, TXsize );
                        argc = 1;
                        continue;
                }
                break;
        }
        return chno;
}


int init_semas()
{
        int isem;

        for( isem=0; isem < NUM_OF_DMA; isem++ ) {
                if( sem_init( &sendSem[isem], 0, 0) == -1) {
                        printf("[%s] Semaphore Init ERROR.\n", __FUNCTION__);
                        return -1;
                }
        }
}


int destroy_semas()
{
        int isem;

        for( isem=0; isem < NUM_OF_DMA; isem++ )
                sem_destroy( &sendSem[isem] );
}


int main(int argc, char **argv)
{
	int chNum[4] = {0, 1, 2, 3};
	int semIndex = 0;
	int chno;

	pthread_t p_gpio;
	pthread_t p_send;
	pthread_t p_recv;

	int iGpioThreadID;
	int iSendThreadID;
	int iRecvThreadID;


	chno = get_main_arg( argc, argv );

	printf( "Execute: %s %d(ch#) %d(TXsize) through SFPDP loopback..\n", argv[0], chno, TXsize );

	/* First, GPIO initialize */
	scGpioInit();
	init_semas();

	if( hdlc_ch_open(chno) != 0 )				return -1;
//	if( hdlc_reg_init( chno, RXMODE_AFTER_PATTERN ) != 0 )	return -1;
	hdlc_bdbuf_clear( chno);
	printf("AIM send & recv loopback start ..\n");

	iGpioThreadID = pthread_create( &p_gpio, NULL, gpio_thread, NULL);
	iSendThreadID = pthread_create( &p_send, NULL, send_thread, (void *)&chno );
	iRecvThreadID = pthread_create( &p_recv, NULL, recv_thread, (void *)&chno );

	pthread_join(p_recv, NULL);
	pthread_join(p_send, NULL);
	pthread_join(p_gpio, NULL);

	scGpioClose();
	destroy_semas();
	hdlc_ch_close(chno);

	return 0;
}


