/*
 * Recv message through an HDLC card. 
 *
 */
#include <stdio.h>
#include <stdlib.h>		// atoi()
#include <pthread.h>
#include <string.h>		// memcpy()
#include <unistd.h>		// read, write
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <time.h>
#include <semaphore.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/poll.h>

#include "gefcmn_gpio.h"
#include "hdlc-api.h"


extern int scGpioInit();
extern int scGpioClose();
extern int scGpioSetHigh7();
extern int scGpioSetLow7();

extern GEF_GPIO_HDL gpioHdl;

static int TXsize=4096;


void *recv_thread(void* arg)
{
	int ch=0, i;
	int nIndex;
	int cntblk=0, retlen; 
	int totrecv=0;
	unsigned char	rbuf[RX_BUFSIZE+16];
	unsigned int	calchksum, rxchksum;
	struct timeval mytime; 

	ch = *(int *)arg;

	while(1) {

		retlen = hdlc_recv( ch, (char *)rbuf, TXsize, 0 );	// timeout: usec
/*		gettimeofday( &mytime, NULL ); 
		printf("\nCurr time(usec) in RX_user: %ld \n", mytime.tv_usec );
*/
		scGpioSetHigh7();

		if( retlen<0 ) {
			printf("\nERR: channel link RX err, retlen: %d\n", retlen );
			return NULL; 
		}
#ifdef DEBUG
		printf("\nCH[%d] RX size: %d Bytes\n", ch, retlen );
		printf("rbuf[0] =0x%02X, rbuf[1] =0x%02X ", rbuf[0], rbuf[1] );
		printf("rbuf[4]cnt =0x%02X \n", rbuf[4]);
#endif
		// to calcurate check-sum of RX data
		
		calchksum = 0;
		for(i=5; i< (retlen-sizeof(int)); i++)
			calchksum += rbuf[i];
		
		memcpy( (void*)&rxchksum, (void*)&rbuf[retlen-sizeof(int)], sizeof(int) );
		if( calchksum != rxchksum ) 
			printf(">>>Err: Calculated chksum=0x%X, RX chksum=0x%X\n", calchksum, rxchksum); 

		totrecv += retlen; 
		scGpioSetLow7();

#ifdef DEBUG
		printf("Ch[%d] RX Total size: %d Bytes\n", ch, totrecv );
		printf("Calculated chksum: 0x%X, RX chksum: 0x%X\n", calchksum, rxchksum ); 
#endif
	}

	return NULL;
}


int get_main_arg( int argc, char **argv )
{
	int chno;

	if( argc != 3 ) printf( "Usage: %s [ch#] [TXsize]\n\n", argv[0] );
	if( argc > 3 ) return -1;

	while(1) {
		if( argc == 1 ) {
			printf( "Select the channel# and size to recieve data..\n" );
			printf( "Range of the Channel number is from 0 to %d: ", NUM_OF_DMA );
			scanf( "%d", &chno );
			printf( "Data Size to recv (5 ~ %d Byte): ", RX_BUFSIZE );
			scanf( "%d", &TXsize );
		}
		else if( argc == 2 ) {
			chno = atoi( argv[1] );
			printf( "Data Size to recv (5 ~ %d Byte): ", RX_BUFSIZE );
			scanf( "%d", &TXsize );
		}
		else if( argc == 3 ) {
			chno = atoi( argv[1] ); 
			TXsize = atoi( argv[2] ); 
		}

		if( (chno<0) || (chno>NUM_OF_DMA) || (TXsize<0) || (TXsize >RX_BUFSIZE) ) {
			printf( "Range of Input is over: Ch[%d], size:%d\n", chno, TXsize );
			argc = 1;
			continue; 
		}
		break; 
	}
	return chno;
}



int main(int argc, char **argv)
{
	int semIndex = 0;
	int chno;
	int irecvID;
	pthread_t p_recv;

	chno = get_main_arg( argc, argv );

	printf( "Execute: %s ch[%d] %d(TXsize) ..\n", argv[0], chno, TXsize );

	/* First, GPIO initialize */
	scGpioInit( );

	if( hdlc_ch_open( chno) != 0 )				return -1;
//	if( aim_reg_init( chno, RXMODE_AFTER_PATTERN ) != 0 )	return -1;
//	hdlc_bdbuf_clear( chno);
	printf("RX start ..\n");

	irecvID = pthread_create( &p_recv, NULL, recv_thread, (void *)&chno );

	pthread_join(p_recv, NULL);

	/* Finalize */
	scGpioClose( );
	hdlc_ch_close( chno);

	return 0;
}

