/*
 * send & recv message through an HDLC board. 
 *
 */
#include <stdio.h>
#include <stdlib.h>		// atoi()
#include <pthread.h>
#include <string.h>		// memcpy()
#include <unistd.h>		// read, write, usleep()
#include <fcntl.h>		// open, close, O_RDWR, O_NONBLOCK
#include <time.h>
#include <semaphore.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/poll.h>

#include "gefcmn_gpio.h"
#include "hdlc-api.h"


extern int scGpioInit();
extern int scGpioSet();
extern int scGpioClose();
extern int scGpioSetHigh6();
extern int scGpioSetLow6();

extern GEF_GPIO_HDL gpioHdl;

static int TXsize=4096;

sem_t sendSem[NUM_OF_DMA];


void *gpio_ch_thread(void* arg)
{
	GEF_GPIO_PIN_BITMAP pinBitmap;
	int i;
	static int gnum = 0;		// Intrpt GPIO input pin #0: MCP
	int ch = *(int *)arg;

#ifdef DEBUG
	GEF_STATUS gret; 
	struct timeval timeout;

	timeout.tv_sec = 2;	// timeout is 3 seconds
	timeout.tv_usec = 0;
#endif

	pinBitmap = (0x0001 << gnum );

	while(1) {
		getchar();

		for(i=0; i<1; i++) {
#ifdef DEBUG
			printf("GPIO Pinmap: 0x%08X.\n", pinBitmap );
			gret = gefGpioWaitForInterrupt( gpioHdl, &pinBitmap, &timeout );
			if( gret == GEF_STATUS_SUCCESS ) 
				printf("GPIO(pin#%d) Interrupted..\n", gnum );
			if( gret == GEF_STATUS_TIMED_OUT ) 
				printf("GPIO(pin#%d) Timed out..\n", gnum );
#else
			gefGpioWaitForInterrupt( gpioHdl, &pinBitmap, NULL );
#endif
		}

		sem_post( &sendSem[ch] );

//		for( i=0; i < NUM_OF_DMA; i++ )	sem_post( &sendSem[i] );
//		usleep(600*9+200);
	}
	return NULL;
}



/* It sends the data with ? size through the Ch (="arg"). */
void *send_thread(void* arg)
{
	int ch=0;
	int i, rtn=0, cnt = 0;
	int totsend=0;			// total size of send
	unsigned int	chksum=0;	// for packet data integrity 
	unsigned char	tbuf[TX_BUFSIZE+64];
	int count = 0;
//	struct timeval mytime; 

	ch = *(int *)arg;
	srand( time(NULL) );

	while(1) {
		chksum = 0;
		tbuf[0] = 0x55;
		tbuf[1] = 0x55;
		tbuf[2] = 0xAA;
		tbuf[3] = 0xAA;
		tbuf[4] =cnt;

		for(i=5; i < (TXsize-sizeof(int)); i++) {
//			tbuf[i] = rand() % 10;
			tbuf[i] = i % 256;
			chksum += tbuf[i];
		}

		memcpy( (void*)&tbuf[i], (void*)&chksum, sizeof(int) );

		sem_wait( &sendSem[ch] );
		scGpioSetHigh6();

		rtn = hdlc_send( ch, (char *)tbuf, TXsize );
/*		gettimeofday( &mytime, NULL );
		printf( "\n Curr time (usec) TX_user: %ld \n", mytime.tv_usec );
*/
#ifdef DEBUG
		if( !(0<rtn) ) {
			printf( "%d-th block, there is no data of hdlc_send.\n", cnt );
		}
		printf( "CH[%d] real TX size:%d Bytes (Req Txsize:%d)\n", ch, rtn, TXsize );
#endif
		totsend += rtn;

		scGpioSetLow6();
#ifdef DEBUG
		printf("Ch[%d] Total TX Size: %d Bytes\n", ch, totsend );
		printf("Ch[%d] TX Check-sum value: 0x%X\n", ch, chksum );
#endif
	} // while

	return NULL;
}


int get_main_args( int argc, char **argv )
{
	int chno;

	if( argc != 3 ) printf( "Usage: %s [ch#] [TXsize]\n\n", argv[0] );
	if( argc > 3 ) return -1;

	while(1) {
		if( argc == 1 ) {
			printf( "Select the channel and size to transfer data..\n" );
			printf( "Range of the Channel number is from 0 to %d: ", NUM_OF_DMA );
			scanf( "%d", &chno );

			printf( "Size of Tx data (5 ~ %d Byte): ", TX_BUFSIZE );
			scanf( "%d", &TXsize );
		}
		else if( argc == 2 ) {
			chno = atoi( argv[1] );
			printf( "Size of Tx data (5 ~ %d Byte): ", TX_BUFSIZE );
			scanf( "%d", &TXsize );
		}
		else if( argc == 3 ) {
			chno = atoi( argv[1] ); 
			TXsize = atoi( argv[2] ); 
		}

		if( (chno<0) || (chno>NUM_OF_DMA) || (TXsize <0) || (TXsize >TX_BUFSIZE) ) {
			printf( "Range of the data is over: %d\n", TXsize );
			argc =1;
			continue; 
		}
		else break;
	}
	return chno;
}


int init_semas()
{
	int isem; 

	for( isem=0; isem < NUM_OF_DMA; isem++ ) {
		if( sem_init( &sendSem[isem], 0, 0) == -1) {
			printf("[%s] Semaphore Init ERROR.\n", __FUNCTION__);
			return -1;
		}
	}
}


int destroy_semas()
{
	int isem; 

	for( isem=0; isem < NUM_OF_DMA; isem++ )
		sem_destroy( &sendSem[isem] );
}



int main(int argc, char **argv)
{
	int isem = 0;
	int chno;

	pthread_t p_gpio;
	pthread_t p_send;

	int igpioID;
	int isendID;

	chno = get_main_args( argc, argv );

	printf( "Execute: %s Ch[%d] %d(TXsize) ..\n", argv[0], chno, TXsize );

	/* First, GPIO initialize */
	scGpioInit();	// using gpio #gnum as interrupt.
	init_semas();

	if( hdlc_ch_open(chno) != 0 )			return -1;
//	if( hdlc_change_txmode( chno, 0 ) != 0 )	return -1;
//	hdlc_bdbuf_clear( chno);

	igpioID = pthread_create( &p_gpio, NULL, gpio_ch_thread, (void *)&chno );
	isendID = pthread_create( &p_send, NULL, send_thread, (void *)&chno );

	printf("TX start ..\n");

	pthread_join(p_send, NULL);
	pthread_join(p_gpio, NULL);

	/* Finalize */
	scGpioClose();
	destroy_semas();

	hdlc_ch_close(chno);

	return 0;
}


