/*
 * This is basic functions for HDLC communication. 
 */
#include <stdio.h>
#include <string.h>	     // strlen()
#include <unistd.h>	     // read, write
#include <fcntl.h>	      // open, close, O_RDWR, O_NONBLOCK
#include <sys/ioctl.h>
#include <sys/poll.h>

#include "hdlc-api.h"


int fd[4]={0,0,0,0};	/* device file id */
/*
int temp_size[4] = {0,0,0,0};
char temp[4][RX_FRMSIZE];
int recv_size[4] = {0,0,0,0};
*/

int hdlc_start(int ch, int mode)
{
	PCIDRV_INFO pi;

	pi.addr = 0x50000 + (0x10000*ch) + 0x18;
	if( mode == RXMODE_AFTER_PATTERN )	pi.val = 0x03600003; //After PATTERN, data transfered. 
	else if( mode == RXMODE_WITHOUT_PATTERN ) pi.val = 0x02600003; // all the data is transfered.
	else return -1;
 
//	pi.val = 0x02600003; 
	ioctl( fd[ch], IOCTL_SET_REG, &pi);

//	printf("0x%x \n",pi.val);
/*

 * 0x4C : Timeout Reg. Between packets the timeout is allowed. 
 * 0x8fc=2300d, Default value is 0x47e (=1150 = 5.75us).
 * if (packet delay > Timeout) at the end of Data, "DEAD_BEEF" is inserted. 
*/
	pi.addr = 0x50000 + (0x10000*ch) + 0x4c;
	pi.val = 0x8fc;
	ioctl( fd[ch], IOCTL_SET_REG, &pi);

//	pi.addr = 0x50000 + (0x10000*ch) + 0x0C;
//	pi.val = 0;		// Mormal operation.
//	ioctl(fd[3], IOCTL_SET_REG, &pi);

	return 0;
}


/* It clears BD buffer in which some garbages remainded */
int hdlc_bdbuf_clear( int ch )
{
	int ret;
	int utimeout =20;       // micro sec.
	char buf[RX_FRMSIZE+16];

	if( !(0<fd[ch]) ) {
		printf( "[%s] ch[%d]: fd is not opened..", __func__, ch );
		return -1;
	}

	/* to extract some garbage data from DMA buffers */
	while(1) {

		ret = read( fd[ch], buf, utimeout );
		if( ret<0 ) {
			printf("[%s] ch[%d] Read_error.\n", __func__, ch );
			return -1;
		}
		if( ret>0 ) continue;   // garbages remained.
		break;
	}
	return 0;
}


/* return: the size of real send data */
int hdlc_send( int ch, char* buf, int size )
{
	int rtn;
	int idx = 0;	// the number of write() function

	if( !(0<fd[ch]) ) {
		printf( "fd[%d] is not opened in [%s]..", ch, __FUNCTION__ );
		return -1;
	}

	while( size > 0 )
	{
		/*JKK: this write() (=pci_write()) return 0 when it successes, return -1 when fail.
 		Originally write() return the size of the real written data. */

		rtn = write( fd[ch], &buf[(TX_FRMSIZE)*idx], (size>TX_FRMSIZE ? TX_FRMSIZE : size) );
		if( rtn < 0) {
			printf( "[%s] write error in.\n", __FUNCTION__ ); 
			return -1;
		}
		size -= TX_FRMSIZE;
		idx++;
	}

	return ( TX_FRMSIZE * idx + size);	// size is now negative.
}


int hdlc_recv_single( int ch, char* buf, int size )
{
	char tbuf[RX_FRMSIZE+64];
	int ret; 


//	ret = read( fd[ch], buf, RX_FRMSIZE);
	ret = read( fd[ch], tbuf, 0 );

	if( ret<0 ) {
		printf("[%s] Read error.\n", __FUNCTION__ );
		return -1;
	}

	memcpy( buf, tbuf, ret );
	return ret; 
}


int hdlc_recv( int ch, char* buf, int size, int usrtimeout)
{
	int ret, i = 0;
	int recv_size=0;
	char tbuf[RX_FRMSIZE+16];

	if( !(0<fd[ch]) ) {
		printf( "fd[%d] is not opened in [%s]..", ch, __func__ );
		return -1;
	}

	while( size > 0) {
		ret = read( fd[ch], tbuf, usrtimeout );
		if( ret<0 ) {
			printf("[%s] Read error.\n", __func__ );
			return -1;
		}
		if( ret==0 ) break;

		memcpy( &buf[recv_size], tbuf, ret );
		recv_size += ret;
		size -= ret;
	}

//      printf("[Amount of Recv(%d)]\n", recv_size );
	return recv_size;
}



/* It receives data with "size" or less into the "buf". 
 * return: the size of real data be received. */
int hdlc_poll( int ch, char* buf, int size, int utimeout)
{
	int ret, i = 0;
	int recv_size=0;
	char rbuf[RX_FRMSIZE+16];
	int pollst;
	struct pollfd poll_events;

	if( !(0<fd[ch]) ) {
		printf( "fd[%d] is not opened in [%s]..", ch, __FUNCTION__ );
		return -1;
	}

	poll_events.fd	= fd[ch];
	poll_events.events    = POLLIN | POLLERR;       // requested events: 수신 자료 or 에러가 있는지
	poll_events.revents   = 0;		      // returned events.
	
	while( size > 0) {
		pollst = poll( (struct pollfd*)&poll_events, 1, utimeout ); // utimeout: millisec.
		if( pollst == 0 ) {	// timeout.
			printf("[%s] Poll is timeout.\n", __func__ );
			continue;
		}

		if( poll_events.revents & POLLIN) {
			ret = read( fd[ch], rbuf, size );
			if( ret<0 ) {
				printf("[%s] Read error.\n", __func__ );
				return -1;
			}
			if( ret==0 ) break; 
		}
		else if( poll_events.revents & POLLERR ) {      // event is error. 
			printf( "[%s] poll_events error..\n", __FUNCTION__ );
			return -1;
		}

		memcpy( &buf[recv_size], rbuf, ret );
		recv_size += ret;
		size -= ret; 
	}

//	printf("[Amount of Recv(%d)]\n", recv_size );
	return recv_size; 
}


int hdlc_init()
{
}


int hdlc_ch_open( int ch )
{
	char devfile[128];

	sprintf( devfile, "/dev/%s%d", HDLC_DEV_NAME, ch );
	fd[ch] = open( devfile, O_RDWR|O_SYNC );
	if( !(0<fd[ch]) ) {
		printf( "[%s] Dev file open() fail: %s.\n", __func__, devfile );
		return -1;
	}
//      printf( "Device file(%s) open for Ch[%d].. OK!\n", devfile, ch );
	return 0;
}


int hdlc_ch_close( int ch )
{
	close( fd[ch] );
}


int hdlc_all_open()
{
	int ich;
	char devfile[128];

	for( ich=0; ich<NUM_OF_DMA; ich++) {
		sprintf( devfile, "/dev/%s%d", HDLC_DEV_NAME, ich );
		fd[ich] = open( devfile, O_RDWR|O_SYNC );
		if( !(0<fd[ich]) ) {
			printf( "[%s] Dev file open() fail: %s.\n", __func__, devfile );
			return -1;
		}
	}
//      printf( "Devices: %d files open(... %s) OK!\n", ich, devfile );
	return 0;
}


int hdlc_all_close()
{
	int ich =0;

	for( ich=0; ich<NUM_OF_DMA; ich++)	close(fd[ich]);
	return 0;
}


int hdlc_reset(int ch)
{
	PCIDRV_INFO pi;

	if( !(0<fd[ch]) ) {
		printf( "fd[%d] is not opened in [%s]..", ch, __FUNCTION__ );
		return -1;
	}

	pi.addr = 0x50000 + (0x10000*ch) + 0x18;
	pi.val = 0x00600003;
	ioctl( fd[ch], IOCTL_SET_REG, &pi);
	usleep(1000*100);

	pi.addr = 0x50000 + (0x10000*ch) + 0x0c;	// Master Control Register
	pi.val = 0x01;					// Reset the sFPDP channel
	ioctl( fd[ch], IOCTL_SET_REG, &pi);
	usleep(1000*100);

	pi.addr = 0x50000 + (0x10000*ch) + 0x0c;
	pi.val = 0x00;					// Normal operations
	ioctl( fd[ch], IOCTL_SET_REG, &pi);
	usleep(1000*100);

	return 0;
}


int hdlc_bd_reset()
{
//	PCIDRV_INFO pi;		// there is no pi setting? 

	if( !(0<fd[3]) ) {
		printf( "fd[%d] is not opened in [%s]..", 3, __FUNCTION__ );
		return -1;
	}

	ioctl( fd[3], IOCTL_RESET, NULL );
	return 0;
}

